import { Component, ViewChild, ElementRef, Renderer, NgZone } from '@angular/core';
import { IonicPage, NavController, NavParams, Platform, LoadingController, ModalController } from 'ionic-angular';
import { GoogleMapsService } from '../../providers/google-maps/google-maps';
import { LocationsService } from '../../providers/locations-service/locations-service';
import { Observable } from 'rxjs/Observable';
import { forkJoin } from 'rxjs/Observable/forkJoin';
import { LatLng, GoogleMapsEvent, Marker } from '@ionic-native/google-maps';
import { Subscription } from 'rxjs/Subscription';
import { AuthService } from '../../providers/auth-service/auth-service';
import { Place } from '../../models/place/place';
import { AngularFirestore } from 'angularfire2/firestore';
import { PlacesService } from '../../providers/places-service/places-service';
/**
 * Generated class for the MapPage page.
 *
 * See https://ionicframework.com/docs/components/#navigation for more info on
 * Ionic pages and navigation.
 */

@IonicPage()
@Component({
  selector: 'page-map',
  templateUrl: 'map.html',
})
export class MapPage {
  @ViewChild('map') mapElement: ElementRef;
  @ViewChild('pleaseConnect') pleaseConnect: ElementRef;
  @ViewChild('input') input: ElementRef;
  @ViewChild('infoWindow') infoWindow: ElementRef;
  @ViewChild('infoWindowNewPoint') infoWindowNewPoint: ElementRef;

  query: string = '';
  cities: any = [];

  infoWindowIsVisible: boolean = false;

  selectedMarker: any = {
    title: null,
    newPlace: null,
    place: null
  };

  newPlace: LatLng;

  clickSubscription: Subscription;

  titleInfoWindow: string = 'Sense títol';

  intervalChangeInfoWindow: number = 300;

  places: Place[];

  constructor(
    public navCtrl: NavController, 
    public maps: GoogleMapsService,
    public platform: Platform,
    public locations: LocationsService,
    public loadingCtrl: LoadingController,
    public auth: AuthService,
    public modalCtrl: ModalController,
    public afs: AngularFirestore,
    public placesService: PlacesService
  ) 
    {
      
  }

  ionViewDidLoad() {
    this.disableAllInfoWindow();
    this.platform.ready().then( () => {
      const loader = this.loadingCtrl.create();
      loader.present();

      let mapLoaded = this.maps.init(this.mapElement.nativeElement, this.pleaseConnect.nativeElement, this.input['_searchbarInput'].nativeElement);
      let locationsLoaded = this.placesService.getPlaces();

      mapLoaded.subscribe( () => {

        locationsLoaded.then( (places) => {

          loader.dismiss();

          this.addMapListeners();

          this.addPlaces(places);       

        }, (error) => {
          loader.dismiss();
        })

      }, (error) => {
        loader.dismiss();
      })
     
    })
  }

  private addMapListeners() {
    this.addMapClickListener();
    this.addMapLongClickListener();
  }


  private addMapClickListener() {
    
    this.clickSubscription = this.maps.clickPosition$.subscribe( (latLng) => {
      this.maps.removeActiveMarker();
      if( this.infoWindowIsVisible) {
        this.disableAllInfoWindow();
      };
    })
  }


  private addMapLongClickListener() {
    this.maps.map.on( GoogleMapsEvent.MAP_LONG_CLICK ).subscribe( (_latLng) => {
      this.maps.removeActiveMarker();
      this.newPlace = _latLng;
      this.maps.addMarkerPoint(_latLng).then( (marker: any) => {
        this.showInfoMarkerPoint(marker);
        this.markerClickListener(marker);
      })
    });
  }


  
  private addPlaces(places) {
    let loader = this.loadingCtrl.create();
    loader.present();

    this.places = [];

    this.clearPage();
   
    places.forEach( (place) => {
      this.addMarker(place, place.uid);
    });

    loader.dismiss();
  }



  private clearPage() {
    this.maps.clearMarkers();
    this.disableAllInfoWindow();
  }




  private addMarker(place: Place, index: string) {
    this.maps.addMarker(place.location.lat, place.location.lng, place.name, index).then(
      (marker) => {
        place.setMarker(marker, this.maps.map);
        this.places.push(place);

        this.markerClickListener(place);
      }
    )
  }




  private markerClickListener(place: Place | Marker){
    let marker = place instanceof Place ? place.marker : place;
    marker.one(GoogleMapsEvent.MARKER_CLICK).then( () => {
      let indexMarker = marker.get('index');
      if ( indexMarker == -1 ) {
        this.showInfoMarkerPoint(marker);
      } else {
        this.showInfoPlace(indexMarker, place);
      }
     
      this.markerClickListener(place);
    });
  }

  


  private showInfoMarkerPoint(marker) {
    if( this.infoWindowIsVisible) {
      this.disableAllInfoWindow();
    };
    setTimeout(() => {
      this.enableInfoMarkerPoint(marker);
    }, this.intervalChangeInfoWindow);
  }




  private showInfoPlace(index, place) {
    let marker = this.maps.getMarker(index);
    this.maps.activeMarker = marker;
    this.maps.removePointMarker();
    if( this.infoWindowIsVisible) {
      this.disableAllInfoWindow();
      setTimeout(() => {
        this.enableInfoWindow(marker, place);
      }, this.intervalChangeInfoWindow);
    } else {
      this.enableInfoWindow(marker, place);
    }
  }


  private enableInfoMarkerPoint(marker) {
    this.infoWindowIsVisible = true;
    this.selectedMarker.title = marker.getTitle();
    this.selectedMarker.newPlace = marker.get('newPlace');
    if (this.infoWindowNewPoint){
      this.infoWindowNewPoint.nativeElement.style.bottom = '0';
    }
  }


  private enableInfoWindow(marker, place) {
    this.infoWindowIsVisible = true;
    this.selectedMarker.title = marker.getTitle();
    this.selectedMarker.newPlace = marker.get('newPlace');
    this.selectedMarker.place = place;
    if (this.infoWindow) {
      this.infoWindow.nativeElement.style.bottom = '0';
    }
  }




  private disableInfoWindow() {
    if (this.infoWindow) {
      this.infoWindow.nativeElement.style.bottom = '-200px';
    }
    this.infoWindowIsVisible = false;
    this.selectedMarker = {
      title: null,
      newPlace: null,
      place: null
    };
  }




  private disableInfoWindowNewPoint() {
    if (this.infoWindowNewPoint){
      this.infoWindowNewPoint.nativeElement.style.bottom = '-200px';
    }
    this.infoWindowIsVisible = false;
  }




  private disableAllInfoWindow() {
    this.disableInfoWindow();
    this.disableInfoWindowNewPoint();
  }




  goToDetail(selectedMarker) {
    console.log(selectedMarker);

    this.navCtrl.push('DetailPlacePage', {item: selectedMarker.place});
  }




  refreshPlaces() {

    const loader = this.loadingCtrl.create();
    loader.present();
    this.maps.map.clear();
    this.placesService.getPlaces().then( (querySnapshot) => {
      loader.dismiss();
      this.addPlaces(querySnapshot);
    })
  }




  addPlace() {
    if ( this.newPlace ) {
        if( !this.auth.authenticated ){
            this.getLogin().subscribe( () => {
              this.goToNewPlace();
            })
        } else {
          this.goToNewPlace();
        }
    }
  }




  getLogin(): Observable<any> {

    return new Observable( (observer) => {
        const modal = this.modalCtrl.create('LoginPage');
        modal.onDidDismiss( (data) => {
            if ( data ) {
              observer.next(data);
            }
            observer.complete();
        })
        modal.present();
    })
  }




  goToNewPlace() {
    const modal = this.modalCtrl.create('NewPlacePage');
    modal.onDidDismiss( (_place) => {
      if (_place) {
        setTimeout( () => {
          this.maps.pointMarker.off(GoogleMapsEvent.MARKER_CLICK);
          let place = new Place();
          place.setData(_place);
          this.addMarker(place, place.uid);          

          this.disableInfoWindowNewPoint();
        }, 1000);
      }
    });
    modal.present();
  }




  selectCity(city) {
    this.cities = [];
    this.maps.selectCity(city);
  }





  searchCity() {
    this.maps.searchCity(this.query, (cities) => {
      this.cities = cities;
    })
  }

  

  openFilters() {
  }



  ionViewWillUnload() {
    if ( this.clickSubscription ){
        this.clickSubscription.unsubscribe();
    }
  }




}
