import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { Observable } from 'rxjs/Observable';
import { AngularFireAuth } from 'angularfire2/auth';
import * as firebase from 'firebase/app';
import AuthProvider = firebase.auth.AuthProvider;
import { AngularFirestore, AngularFirestoreCollection, AngularFirestoreDocument } from 'angularfire2/firestore';
import { UserInterface, User } from '../../models/user/user';
import { GooglePlus } from '@ionic-native/google-plus';


/*
  Generated class for the AuthServiceProvider provider.

  See https://angular.io/guide/dependency-injection for more info on providers
  and Angular DI.
*/
@Injectable()
export class AuthService {

  private userFirebase: firebase.User;

  private user: User;

  private userSubject: BehaviorSubject<User> = new BehaviorSubject<User>(null);

 
  constructor(
    private http: HttpClient, 
    public afAuth: AngularFireAuth, 
    public afs: AngularFirestore,
    public googlePlus: GooglePlus
  ) {
    console.log('Hello AuthService Provider');
    
    afAuth.auth.onAuthStateChanged( (user) => {
      if( user ) {
        this.userFirebase = user;    
        this.getUserData();       
      } else {
        this.userFirebase = null;
        this.user = null;
        this.userSubject.next(null);
      }
      
    })

   

  }

  subscribeUser(): Observable<User> {
    return this.userSubject.asObservable();
  }

  private getUserData() {
    let userRef = this.afs.collection('users').doc(this.userFirebase.uid).valueChanges();
    userRef.subscribe( (user) => {
        this.user = new User();
        this.user.set(user);
        this.userSubject.next(this.user);
    }, (error) => {
      console.error(error);
    })
  }


  getUser(): Promise<User> {
    return new Promise( (resolve, reject) => {
      if ( this.user != null ) {
        resolve(this.user);
      } else if( this.userFirebase != null) {
        return this.getUserData();
      } else {
        resolve(null);
      }
    })
  }



  get authenticated(): boolean {
    return this.user !== null;
  }


  getEmail() {
    return this.user && this.user.email;
  }


  /**
   * Autenticació
   * @param username 
   * @param password 
   */
  public login(email: string, password: string): Promise<any> {
    let credentials = {
			email: email,
			password: password
		};
    return this.afAuth.auth.signInWithEmailAndPassword(credentials.email, credentials.password);
  }


  /**
   * Registre
   * @param username 
   * @param email 
   * @param password 
   */
  public signup(username: string, email: string, password: string): Promise<any> {
    let credentials = {
			email: email,
			password: password
		};
    return this.afAuth.auth.createUserWithEmailAndPassword(credentials.email, credentials.password)
        .then( (user) => {
          return this.updateUserData(user, username);
        });
  }


  signInWithGoogle() {
    console.log('Sign in with google');
    const provider = new firebase.auth.GoogleAuthProvider();
		return this.oauthSignIn(provider);
  }

  private oauthSignIn(provider: AuthProvider) {
    if (!(<any>window).cordova) {
      return this.afAuth.auth.signInWithPopup(provider)
        .then( (credential) => {
            return this.updateUserData( credential.user );
        })
    } else {
      return this.afAuth.auth.signInWithRedirect(provider)
      .then(() => {
          return new Promise( (resolve, reject) => {
              this.afAuth.auth.getRedirectResult().then( credential => {
                  let token = credential.credential.accessToken;
                
                  let user = credential.user;
        
                  resolve(this.updateUserData( credential.user ));
              }).catch(function(error) {
              
                  reject(error);
              });
          })
      });
    }
  }


  private updateUserData(user, username?: string) {
    const userRef: AngularFirestoreDocument<any> = this.afs.doc(`users/${user.uid}`);

    const data: UserInterface = {
      uid: user.uid,
      email: user.email,
      display_name: !user.displayName && username ? username : user.displayName,
      photo_profile: user.photoURL,
      rating: null
    }

    const set = userRef.set(data);
    return true;
  }


  public signOut(): Promise<void> {
    return this.afAuth.auth.signOut();
  }

}
