import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { AngularFirestore } from 'angularfire2/firestore';
import { Observable } from 'rxjs/Observable';
import { Place } from '../../models/place/place';
import { PlaceInterface, EntryRegister, PlaceComment, PlaceRating } from '../utils/constants';
import { AngularFireStorage } from 'angularfire2/storage';
import { storage } from 'firebase/app';
import { AuthService } from '../auth-service/auth-service';
import { User } from '../../models/user/user';
import { Subscription } from 'rxjs/Subscription';
import { UpperCasePipe } from '@angular/common';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';

/*
  Generated class for the PlacesServiceProvider provider.

  See https://angular.io/guide/dependency-injection for more info on providers
  and Angular DI.
*/
@Injectable()
export class PlacesService {

  private currentUser: User;
  
  private userSubscription: Subscription;

  private placesSubject: BehaviorSubject<Place[]> = new BehaviorSubject<Place[]>(null);

  constructor(
    public http: HttpClient,
    public afs: AngularFirestore,
    public afStorage: AngularFireStorage,
    public auth: AuthService
  ) {


    this.userSubscription = this.auth.subscribeUser().subscribe( (user) => {

      if ( user ) {
        this.currentUser = new User();
        this.currentUser.set( user );
      } else {
        this.currentUser = null;
      }
      
    })
  
  }

  subscriptionPlaces() {
    return this.placesSubject.asObservable();
  }

  getPlaces(): Promise<any> {
    let result = [];
    const places = this.afs.collection('places');
    return places.ref.get().then( (querySnapshot) => {
      

      querySnapshot.forEach( async (doc: any) => {
        let place = new Place();
        place.setData(doc.data());
        let gp = this.getPictures(place.uid, place.images).then( (pictureUrls) => {
            place.imagesUrl = pictureUrls;
            return place;
        });
        result.push(gp);
      })
      return new Promise ( (resolve) => {
        Promise.all(result).then( (data) => {
          this.placesSubject.next(data);
          resolve(data);
        });
      });
    })
  }


  getPictures(place_uid: string, pictureIds: string[]): Promise<any> {

    const pictures = storage().ref(`pictures/${place_uid}`);

    let urlPromises = [];
    pictureIds.forEach( (pictureId, index) => {
      let promise = pictures.child(pictureId).getDownloadURL();
      urlPromises.push(promise);
    })
  
    return Promise.all(urlPromises);
      
  }
  


  getPlaceCategories(): Observable<any> {
    const categories = this.afs.collection('place_categories');
    return categories.valueChanges();
  }


  getActivities(): Observable<any> {
    const categories = this.afs.collection('activities');
    return categories.valueChanges();
  }


  save(_place: PlaceInterface): Promise<any> {

   
    return new Promise( (resolve, reject) => {

      const newPlace: PlaceInterface = {
        uid: this.afs.createId(),
        name: _place.name,
        description: _place.description,
        location: Object.assign({}, _place.location),
        place_category: Object.assign({}, _place.place_category),
        place_details: Object.assign({}, _place.place_details),
        activities: _place.activities,
        user: {
          uid: this.currentUser.uid,
          display_name: this.currentUser.display_name,
          photo_profile: this.currentUser.getPhotoProfile(),
          rating: this.currentUser.rating
        },
        created_at: new Date(),
        updated_at: null,
        checkin_registers:[],
        ratings: [],
        comments: [],
        images: [],
        marker: null
      }

      this.savePictures(newPlace.uid, _place.images).then( (pictureIds) => {
          newPlace.images = pictureIds;
  
          this.afs.collection('places').doc(newPlace.uid).set(newPlace).then( (data) => {
  
              this.getPlaces();
              
              let docRef = this.afs.collection('places').doc(newPlace.uid).ref;
              docRef.get().then( (doc) => {
                  resolve(doc.data());
              }, (error) => reject(error));
  
          }, (error) => reject(error));

      }, (error) => reject(error))

    })
  }


  savePictures(place_uid: string, images: string[]): Promise<any> {

    return new Promise( (resolve) => {

      const pictures = this.afStorage.storage.ref(`pictures/${place_uid}`);
      let pictureIds = [];
      images.forEach( (image, index) => {
        let pictureId = this.afs.createId();
        pictureIds.push(pictureId);
        pictures.child(pictureId).putString(image, 'data_url');
      })

      resolve(pictureIds);
    }) 

  }

 

  checkInHere(place: Place): Promise<any> {
      
      let newObj: EntryRegister = {
        created_at: new Date(),
        user: {
          display_name: this.currentUser.display_name,
          photo_profile: this.currentUser.getPhotoProfile(),
          rating: this.currentUser.rating,
          uid: this.currentUser.uid
        }
      };

      if( !place.checkin_registers ) {
        place.checkin_registers = [];
      }
      place.checkin_registers.push(newObj);

      var placeRef = this.afs.collection('places').doc(place.uid);

      return new Promise ( (resolve, reject) => {
        placeRef.set({ checkin_registers: place.checkin_registers, updated_at: new Date() }, { merge: true }).then( () => {
          resolve(place);
        }).catch( (error) => {
          reject(error);
        })
      })
  }


  subscription( placeId: string): Observable<any> {
    return new Observable( (observer) => {
        this.afs.collection('places').doc(placeId).valueChanges().subscribe( (item: any) => {
            this.getPictures(item.uid, item.images).then( (data) => {
                item.imagesUrl = data;
                observer.next(item);
            })
        });
    })
  }


  ratingPlace( place: Place, rating: number, comment?: string): Promise<any> {
      let newObj = {};

      let newRating: PlaceRating = {
          rating: rating,
          user_id: this.currentUser.uid,
          created_at: new Date()
      };

      if( !place.ratings ) {
        place.ratings = [];
      }
      place.ratings.push(newRating);

      newObj['ratings'] = place.ratings;

      if (comment) {
          let newComment: PlaceComment = {
            comment: comment,
            user:  {
              uid: this.currentUser.uid,
              display_name: this.currentUser.display_name,
              photo_profile: this.currentUser.getPhotoProfile(),
              rating: this.currentUser.rating
            },
            rating: rating,
            created_at: new Date()
          };

          if(!place.comments) {
            place.comments = [];
          }
          place.comments.push(newComment);

          newObj['comments'] = place.comments;
      }

      newObj['updated_at'] = new Date();

      var placeRef = this.afs.collection('places').doc(place.uid);

      return placeRef.set(newObj, { merge: true });
  }

  addComment(place: Place, comment: string): Promise<any> {
      let newComment: PlaceComment = {
        comment: comment,
        user:  {
          uid: this.currentUser.uid,
          display_name: this.currentUser.display_name,
          photo_profile: this.currentUser.getPhotoProfile(),
          rating: this.currentUser.rating
        },
        created_at: new Date()
      };

      if(!place.comments) {
        place.comments = [];
      }
      place.comments.push(newComment);

      var placeRef = this.afs.collection('places').doc(place.uid);

      return placeRef.set({comments: place.comments, updated_at: new Date()}, { merge: true });
  }

}
